#include "philosopher.h++"
#include <boost/bind.hpp>

philosopher::philosopher(int id_i,
                         boost::mutex& left_i,
                         boost::mutex& right_i)
  : id(id_i), left(left_i), right(right_i)
{
  s = thinking;
  bellyfull = ( double(rand() % 10) + 10)/100;
  full = false;
  starved = false;
  action = boost::thread(boost::bind(&philosopher::loop, this));
  hunger = boost::thread(boost::bind(&philosopher::hungry, this));
}

void philosopher::busy(int loops)
{
  for(size_t i = 0; i < loops; i++)
  {
    int t = rand() % 1000;
    boost::this_thread::sleep(boost::posix_time::milliseconds(t));
  }
}

void philosopher::eat()
{
  s = eating;
  int mouthfuls = rand() % 5 + 1;
  for(size_t i = 0; i < mouthfuls; i++)
  {
    double fill = (double(rand() % 5) + 1)/100;
    {
      bellyfull += fill;
      if(bellyfull >= 1.00)
      {
        full = true;
        break;
      }
    }
    busy();
  }
}

void philosopher::think()
{
  s = thinking;
  int thoughts = rand() %5 + 1;
  for(size_t i = 0; i < thoughts; i++)
  {
    busy();
  }
}

void philosopher::hungry()
{
  while(not starved and not full)
  {
    busy(3);
    double bellyempty = ( double(rand() % 10) + 10)/100;
    {
      bellyfull -= bellyempty;
      if(bellyfull <= 0)
        starved = true;
    }
  }
}

void philosopher::loop()
{
  while(not full and not starved)
  {
    think();
    {
      busy();
      boost::mutex::scoped_lock(left);
      busy();
      boost::mutex::scoped_lock(right);

      eat();
    }
  }
}

void philosopher::begin()
{
  action.join();
  hunger.join();
}
