#include <iostream>
#include <vector>
#include <cmath>
#include <sstream>

using namespace std;

void list_sample_space(vector<vector<bool> > &out, size_t n)
{
  if(n == 1){
    out.resize(2);
    out[0].resize(1); out[0][0] = false;
    out[1].resize(1); out[1][0] = true;
  }
  else{
    vector<vector<bool> > front, back;
    list_sample_space(front,n-1);
    back = front;
    for(size_t k = 0; k < front.size(); k++){
      front[k].push_back(false);
      back[k] .push_back(true);
    }
    
    out = front;
    out.insert(out.end(), back.begin(), back.end());
  }
}

template<typename T>
ostream& operator<<(ostream& os, const vector<vector<T> >& v)
{
  for(size_t i = 0; i < v.size(); i++){
    for(size_t j = 0; j < v[i].size(); j++)
      os << v[i][j] << " ";
    os << endl;
  }
  return os;
}

vector<size_t> get_streaks(const vector<vector<bool> >& v)
{
  vector<size_t> out(v.size());
  for(size_t i = 0; i < v.size(); i++){
    bool prev = v[i][0];
    size_t longest_streak = 1, this_streak = 1;
    for(size_t j = 1; j < v[i].size(); j++){
      if(v[i][j] == prev){
	this_streak++;
	longest_streak = max(this_streak, longest_streak);
      }
      else{
	prev = v[i][j];
	this_streak = 1;
      }
    }
    out[i] = longest_streak;
  }
  return out;
}

vector<vector<size_t> > tally_streaks(const vector<size_t>& v,
				      size_t n)
{
  vector<vector<size_t> > out(n);
  for(size_t i = 0; i < n; i++){
    out[i].resize(2);
    out[i][0] = i+1;
    out[i][1] = 0;
  }
  for(size_t i = 0; i < v.size(); i++)
    out[v[i]-1][1]++;
  return out;
}

int main(int argc, char** argv){
  //First arg is size of vectors
  size_t n;
  stringstream ss;
  ss << argv[1];
  ss >> n;

  //The sample space goes in here...
  vector<vector<bool> > sample;
  list_sample_space(sample,n);

  //Now get the size of the longest streak on each sample point
  vector<size_t> streaks = get_streaks(sample);

  //Tally up the length of streaks
  vector<vector<size_t> > tally = tally_streaks(streaks,n);
  
  cout << tally;
}
